@@ -1,4 +1,9 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
 import sys, os, glob
 from bup import options, path
 
@@ -21,7 +26,7 @@ elif len(extra) == 1:
             os.execvp('man', ['man', '-l', g[0]])
         else:
             os.execvp('man', ['man', docname])
-    except OSError, e:
+    except OSError as e:
         sys.stderr.write('Unable to run man command: %s\n' % e)
         sys.exit(1)
 else:
