@@ -1,8 +1,14 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import sys
 
 from bup import git, options, client
-from bup.helpers import *
+from bup.helpers import log, saved_errors
 
 
 optspec = """
@@ -19,7 +25,7 @@ if extra:
 
 try:
     git.init_repo()  # local repo
-except git.GitError, e:
+except git.GitError as e:
     log("bup: error: could not init repository: %s" % e)
     sys.exit(1)
 
