@@ -1,7 +1,14 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import sys
+
 from bup import git, options, client
-from bup.helpers import *
+from bup.helpers import linereader, log
 
 
 optspec = """
@@ -36,7 +43,7 @@ for id in extra:
     try:
         for blob in cat(id):
             outfile.write(blob)
-    except KeyError, e:
+    except KeyError as e:
         outfile.flush()
         log('error: %s\n' % e)
         ret = 1
