@@ -1,7 +1,13 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import sys
+
 from bup import git, vfs, ls
-from bup.helpers import *
 
 
 git.check_repo_or_die()
