@@ -1,7 +1,14 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import sys, struct, math
+
 from bup import options, git, _helpers
-from bup.helpers import *
+from bup.helpers import log
 
 POPULATION_OF_EARTH=6.7e9  # as of September, 2010
 
