@@ -1,10 +1,19 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import sys, re, struct, time, resource
+
 from bup import git, bloom, midx, options, _helpers
-from bup.helpers import *
+from bup.helpers import handle_ctrl_c
+
 
 handle_ctrl_c()
 
+
 _linux_warned = 0
 def linux_memstat():
     global _linux_warned
@@ -12,7 +21,7 @@ def linux_memstat():
     d = {}
     try:
         f = open('/proc/self/status')
-    except IOError, e:
+    except IOError as e:
         if not _linux_warned:
             log('Warning: %s\n' % e)
             _linux_warned = 1
