@@ -1,7 +1,15 @@
-#!/usr/bin/env python
-import sys
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
+import os, sys
+
 from bup import options, _helpers
-from bup.helpers import *
+from bup.helpers import atoi, handle_ctrl_c, log, parse_num
+
 
 optspec = """
 bup random [-S seed] <numbytes>
