@@ -1,7 +1,15 @@
-#!/usr/bin/env python
+#!/bin/sh
+"""": # -*-python-*-
+bup_python="$(dirname "$0")/bup-python" || exit $?
+exec "$bup_python" "$0" ${1+"$@"}
+"""
+# end of bup preamble
+
 import os, sys, struct
+
 from bup import options, git
-from bup.helpers import *
+from bup.helpers import Conn, debug1, debug2, linereader, log
+
 
 suspended_w = None
 dumb_server_mode = False
@@ -153,7 +161,7 @@ def cat(conn, id):
         for blob in cat_pipe.join(id):
             conn.write(struct.pack('!I', len(blob)))
             conn.write(blob)
-    except KeyError, e:
+    except KeyError as e:
         log('server: error: %s\n' % e)
         conn.write('\0\0\0\0')
         conn.error(e)
