@@ -1,6 +1,10 @@
-import re, struct, errno, time, zlib
+
+import errno, os, re, struct, sys, time, zlib
+
 from bup import git, ssh
-from bup.helpers import *
+from bup.helpers import (Conn, atomically_replaced_file, chunkyreader, debug1,
+                         debug2, linereader, mkdirp, progress, qprogress)
+
 
 bwlimit = None
 
@@ -75,7 +79,7 @@ class Client:
                     self.pout = self.p.stdout
                     self.pin = self.p.stdin
                     self.conn = Conn(self.pout, self.pin)
-                except OSError, e:
+                except OSError as e:
                     raise ClientError, 'connect: %s' % e, sys.exc_info()[2]
             elif self.protocol == 'bup':
                 self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -94,7 +98,7 @@ class Client:
     def __del__(self):
         try:
             self.close()
-        except IOError, e:
+        except IOError as e:
             if e.errno == errno.EPIPE:
                 pass
             else:
@@ -130,7 +134,7 @@ class Client:
                                   % rv)
         try:
             return self.conn.check_ok()
-        except Exception, e:
+        except Exception as e:
             raise ClientError, e, sys.exc_info()[2]
 
     def check_busy(self):
@@ -300,7 +304,8 @@ class PackWriter_Remote(git.PackWriter):
             self.onopen()
             self._packopen = True
 
-    def _end(self):
+    def _end(self, run_midx=True):
+        assert(run_midx)  # We don't support this via remote yet
         if self._packopen and self.file:
             self.file.write('\0\0\0\0')
             self._packopen = False
@@ -332,7 +337,7 @@ class PackWriter_Remote(git.PackWriter):
         try:
             (self._bwcount, self._bwtime) = _raw_write_bwlimit(
                     self.file, outbuf, self._bwcount, self._bwtime)
-        except IOError, e:
+        except IOError as e:
             raise ClientError, e, sys.exc_info()[2]
         self.outbytes += len(data)
         self.count += 1
