@@ -171,8 +171,11 @@ WVSTART 'metadata save/restore (general)'
     # Check that the only difference is the top dir.
     WVFAIL $TOP/t/compare-trees -c src/var/ src-restore/ > tmp-compare-trees
     WVPASSEQ $(cat tmp-compare-trees | wc -l) 1
-    # Note: OS X rsync itemize output is currently only 9 chars, not 11.
-    expected_diff_rx='^\.d\.\.t.\.\.\.\.?\.? \./$'
+    # The number of rsync status characters varies, so accept any
+    # number of trailing dots.  For example OS X native rsync produces
+    # 9, but Homebrew's produces 12, while on other platforms, 11 is
+    # common.
+    expected_diff_rx='^\.d\.\.t\.\.\.(\.)+ \./$'
     if ! grep -qE "$expected_diff_rx" tmp-compare-trees; then
         echo -n 'tmp-compare-trees: ' 1>&2
         cat tmp-compare-trees 1>&2
@@ -417,13 +420,13 @@ WVSTART 'meta --no-recurse'
     WVPASS mkdir src/foo
     WVPASS touch src/foo/{1,2,3}
     WVPASS bup meta -cf src.meta src
-    WVPASSEQ "$(LC_ALL=C; bup meta -tf src.meta | sort)" "src/
+    WVPASSEQ "$(bup meta -tf src.meta | LC_ALL=C sort)" "src/
 src/foo/
 src/foo/1
 src/foo/2
 src/foo/3"
     WVPASS bup meta --no-recurse -cf src.meta src
-    WVPASSEQ "$(LC_ALL=C; bup meta -tf src.meta | sort)" "src/"
+    WVPASSEQ "$(bup meta -tf src.meta | LC_ALL=C sort)" "src/"
     WVPASS rm -r "$tmpdir"
 ) || exit $?
 
@@ -435,9 +438,9 @@ src/foo/3"
 
     tmpdir="$(WVPASS wvmktempdir)" || exit $?    
 
-    first_group="$(WVPASS python -c 'import os,grp; \
+    first_group="$(WVPASS bup-python -c 'import os,grp; \
       print grp.getgrgid(os.getgroups()[0])[0]')" || exit $?
-    last_group="$(python -c 'import os,grp; \
+    last_group="$(bup-python -c 'import os,grp; \
       print grp.getgrgid(os.getgroups()[-1])[0]')" || exit $?
     last_group_erx="$(escape-erx "$last_group")"
 
@@ -675,7 +678,7 @@ if [ "$root_status" = root ]; then
             WVPASS mkdir "$testfs"/src/foo
             WVPASS touch "$testfs"/src/bar
             PYTHONPATH="$TOP/lib" \
-                WVPASS python -c "from bup import xstat; \
+                WVPASS bup-python -c "from bup import xstat; \
                 x = xstat.timespec_to_nsecs((42, 0));\
                    xstat.utime('$testfs/src/foo', (x, x));\
                    xstat.utime('$testfs/src/bar', (x, x));"
@@ -716,7 +719,7 @@ if [ "$root_status" = root ]; then
                 WVPASS cd "$testfs_limited"/src-restore
                 WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
                     | WVPASS grep -e '^Linux chattr:' \
-                    | WVPASS python -c \
+                    | WVPASS bup-python -c \
                     'import sys; exit(not len(sys.stdin.readlines()) == 3)'
             ) || exit $?
         ) || exit $?
@@ -740,7 +743,7 @@ if [ "$root_status" = root ]; then
             WVFAIL bup meta --extract --file "$testfs"/src.meta
             WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
                 | WVPASS grep -e "^xattr\.set '" \
-                | WVPASS python -c \
+                | WVPASS bup-python -c \
                 'import sys; exit(not len(sys.stdin.readlines()) == 2)'
         ) || exit $?
 
@@ -762,7 +765,7 @@ if [ "$root_status" = root ]; then
             WVPASS cd "$testfs_limited"/src-restore
             WVFAIL bup meta --extract --file "$testfs"/src.meta 2>&1 \
                 | WVPASS grep -e '^POSIX1e ACL applyto:' \
-                | WVPASS python -c \
+                | WVPASS bup-python -c \
                 'import sys; exit(not len(sys.stdin.readlines()) == 2)'
         ) || exit $?
 
